@@ -113,10 +113,14 @@
 #endif
 
 #ifndef TDOM_LS_MODIFIER
-#  ifdef _WIN32
-#    define TDOM_LS_MODIFIER "I64"
-#  else 
-#    define TDOM_LS_MODIFIER "ll"
+#  ifdef XML_LARGE_SIZE
+#    ifdef _WIN32
+#      define TDOM_LS_MODIFIER "I64"
+#    else
+#      define TDOM_LS_MODIFIER "ll"
+#    endif
+#  else
+#    define TDOM_LS_MODIFIER "l"
 #  endif
 #endif
 
@@ -624,7 +628,9 @@ typedef struct domNS {
 } domNS;
 
 
-#define MAX_PREFIX_LEN   80
+#ifndef MAX_PREFIX_LEN
+# define MAX_PREFIX_LEN   80
+#endif
 
 /*---------------------------------------------------------------------------
 |   type domActiveNS
