@@ -24,7 +24,7 @@ typedef enum {
 #define JSON_FALSE 5
 #define JSON_STRING 6
 #define JSON_NUMBER 7
-
+#define JSON_BOOLEAN 8
 
 domDocument *
 JSON_Parse (
@@ -46,3 +46,10 @@ isJSONNumber (
     char *num,
     domLength numlen
     );
+
+int
+jsonEscape (
+    Tcl_Interp *interp,
+    char *str, 
+    Tcl_DString *clearedstr,
+    int *changed);
