@@ -76,6 +76,7 @@ typedef struct NodeInfo {
     char *namespace;
     int   jsonType;
     char *tagName;
+    int   flags;
 } NodeInfo;
 
 /*----------------------------------------------------------------------------
@@ -266,12 +267,16 @@ nodecmd_processAttributes (
     int type,
     int             objc,
     Tcl_Obj *const  objv[],
-    Tcl_Obj **cmdObj
+    Tcl_Obj **cmdObj,
+    int flags
     )
 {
     Tcl_Obj **opts;
     domLength i, len;
-    char *tval, *aval;
+    char *tval, *aval, *p;
+    const char *uri;
+    char **mappings;
+    int maybefq, fq;
     
     /*
      * Allow for following syntax:
@@ -280,7 +285,12 @@ nodecmd_processAttributes (
      *   cmd key_value_list script
      *       where list contains "-key value ..." or "key value ..."
      */
-
+    if (flags & FS_ATT_NO_NAMESPACED) {
+        maybefq = 0;
+    } else {
+        maybefq = 1;
+    }
+    mappings = node->ownerDocument->prefixNSMappings;
     if ((objc % 2) == 0) {
         *cmdObj = objv[objc-1];
         len  = objc - 2; /* skip both command and script */
@@ -304,12 +314,36 @@ nodecmd_processAttributes (
         if (*tval == '-') {
             tval++;
         }
+        uri = NULL;
+        fq = 0;
+        if (maybefq) {
+            p = tval;
+            while (*p) {
+                if (*p == ':') {
+                    fq = 1;
+                    break;
+                }
+                p++;
+            }
+        }
         if (abs(type) == ELEMENT_NODE_ANAME_CHK
             || abs(type) == ELEMENT_NODE_CHK) {
-            if (!tcldom_nameCheck (interp, tval, "attribute", 0)) {
+            if (!tcldom_nameCheck (interp, tval, "attribute", fq)) {
                 return TCL_ERROR;
             }
         }
+        if (fq) {
+            *p = '\0';
+            uri = domLookupPrefixWithMappings (node, tval, mappings);
+            if (!uri) {
+                Tcl_ResetResult (interp);
+                Tcl_AppendResult (interp, "Attribute prefix '", tval,
+                                  "' does not resolve", NULL);
+                *p = ':';
+                return TCL_ERROR;
+            }
+            *p = ':';
+        }
         aval = Tcl_GetString(opts[i+1]);
         if (abs(type) == ELEMENT_NODE_AVALUE_CHK
             || abs(type) == ELEMENT_NODE_CHK) {
@@ -317,7 +351,11 @@ nodecmd_processAttributes (
                 return TCL_ERROR;
             }
         }
-        domSetAttribute(node, tval, aval);
+        if (uri) {
+            domSetAttributeNS (node, tval, aval, uri, 1);
+        } else {
+            domSetAttribute(node, tval, aval);
+        }
     }
     return TCL_OK;
 }
@@ -482,12 +520,16 @@ NodeObjCmd (
 
         cmdObj = NULL;
         if (nodecmd_processAttributes (interp, newNode, type, objc, objv,
-                                         &cmdObj) != TCL_OK) {
+                                       &cmdObj, nodeInfo->flags)
+            != TCL_OK) {
             return TCL_ERROR;
         }
         if (cmdObj) {
             ret = nodecmd_appendFromScript(interp, newNode, cmdObj);
         }
+        if (newNode->firstChild == NULL && nodeInfo->flags & FS_NOT_EMPTY) {
+            domDeleteNode (newNode, NULL, NULL);
+        }
         break;
     }
 
@@ -545,7 +587,7 @@ nodecmd_createNodeCmd (
     int             checkCharData       /* Flag: Data checks? */
 ) {
     int index, ret, type, nodecmd = 0, jsonType = 0, haveJsonType = 0;
-    int isElement = 0;
+    int isElement = 0, flags = 0;
     char *nsName, buf[64];
     Tcl_Obj *tagName = NULL, *namespace = NULL;
     Tcl_DString cmdName;
@@ -567,11 +609,13 @@ nodecmd_createNodeCmd (
     };
 
     static const char *options[] = {
-        "-returnNodeCmd", "-jsonType", "-tagName", "-namespace", NULL
+        "-returnNodeCmd", "-jsonType", "-tagName", "-namespace",
+        "-noNamespacedAttributes", "-notempty", NULL
     };
 
     enum option {
-        o_returnNodeCmd, o_jsonType, o_tagName, o_namespace
+        o_returnNodeCmd, o_jsonType, o_tagName, o_namespace,
+        o_noNamespacedAttributes, o_notempty
     };
 
     static const char *jsonTypes[] = {
@@ -582,7 +626,8 @@ nodecmd_createNodeCmd (
         "TRUE",
         "FALSE",
         "STRING",
-        "NUMBER"
+        "NUMBER",
+        "BOOLEAN"
     };
 
     if (objc < 3 ) {
@@ -622,7 +667,18 @@ nodecmd_createNodeCmd (
             objc -= 2;
             objv += 2;
             break;
-            
+
+        case o_noNamespacedAttributes:
+            flags  |= FS_ATT_NO_NAMESPACED;
+            objc--;
+            objv++;
+            break;
+
+        case o_notempty:
+            flags  |= FS_NOT_EMPTY;
+            objc--;
+            objv++;
+            break;
         }
     }
     if (objc != 3) {
@@ -693,7 +749,7 @@ nodecmd_createNodeCmd (
             if (jsonType < 3 && jsonType > 0) {
                 Tcl_SetResult(interp, "For a text node the jsonType "
                               "argument must be one out of this list: "
-                              "TRUE FALSE NULL NUMBER STRING NONE",
+                              "TRUE FALSE NULL NUMBER STRING NONE or BOOLEAN",
                               NULL);
                 return TCL_ERROR;
             }
@@ -739,7 +795,13 @@ nodecmd_createNodeCmd (
     if (namespace && !isElement) {
         Tcl_SetResult(interp, "The -namespace option is allowed only for "
                       "element node commands.", NULL);
-        return TCL_ERROR;        
+        return TCL_ERROR;
+    }
+
+    if ((flags & ~FS_ATT_NO_NAMESPACED) && !isElement) {
+        Tcl_SetResult(interp, "The -noNamespacedAttributes option is allowed "
+                      "only for element node commands.", NULL);
+        return TCL_ERROR;
     }
     
     if (haveJsonType && type != ELEMENT_NODE && type != TEXT_NODE) {
@@ -750,6 +812,7 @@ nodecmd_createNodeCmd (
     
     nodeInfo = (NodeInfo *) MALLOC (sizeof (NodeInfo));
     nodeInfo->namespace = NULL;
+    nodeInfo->flags = flags;
     nodeInfo->type = type;
     if (nodecmd) {
         nodeInfo->type *= -1; /* Signal this fact */
