@@ -2,4 +2,4 @@
 domDocument *  XML_SimpleParseDocument ( char *xml, int ignoreWhiteSpaces,
                                          int keepCDATA, int forest,
                                          char *baseURI, Tcl_Obj *extResolver,
-                                         int *pos, char **errStr );
+                                         domLength *pos, char **errStr );
