@@ -7,11 +7,11 @@
 #
 
 package require Tcl 8.5-
-package require tcltest 2.2
+package require tcltest 2.2-
 namespace import ::tcltest::*
 catch {tcltest::loadTestedCommands}
 
-if {[catch {package require -exact tdom 0.9.5}]} {
+if {[catch {package require -exact tdom 0.9.6}]} {
     if {[package vsatisfies [package provide Tcl] 9.0-]} {
         set libname libtcl9tdom0.9.5[info sharedlibextension]
     } else {
@@ -34,3 +34,5 @@ if {[package vsatisfies [package present Tcl] 9]} {
     testConstraint Tcl9 1
 }
 testConstraint 64bit [expr {$tcl_platform(pointerSize) >= 8}]
+testConstraint groklargedata [dom featureinfo largedata]
+
